@@ -12,15 +12,21 @@ import shutil
 import threading
 import urllib
 import urllib.parse
-from collections.abc import Generator, Mapping, MutableMapping, MutableSequence
+from collections.abc import (
+    Generator,
+    Iterable,
+    Mapping,
+    MutableMapping,
+    MutableSequence,
+)
 from enum import Enum
 from functools import cmp_to_key, partial
 from re import Pattern
-from typing import TYPE_CHECKING, Any, Iterable, Optional, TextIO, Union, cast
+from typing import TYPE_CHECKING, Any, Optional, TextIO, Union, cast
 
 from mypy_extensions import mypyc_attr
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
-from schema_salad.avro.schema import Schema
+from schema_salad.avro.schema import RecordSchema
 from schema_salad.exceptions import ValidationException
 from schema_salad.ref_resolver import file_uri, uri_file_path
 from schema_salad.sourceline import SourceLine
@@ -148,11 +154,11 @@ class ExpressionJob:
         self,
         builder: Builder,
         script: str,
-        output_callback: Optional[OutputCallbackType],
+        output_callback: OutputCallbackType | None,
         requirements: list[CWLObjectType],
         hints: list[CWLObjectType],
-        outdir: Optional[str] = None,
-        tmpdir: Optional[str] = None,
+        outdir: str | None = None,
+        tmpdir: str | None = None,
     ) -> None:
         """Initialize this ExpressionJob."""
         self.builder = builder
@@ -167,7 +173,7 @@ class ExpressionJob:
     def run(
         self,
         runtimeContext: RuntimeContext,
-        tmpdir_lock: Optional[threading.Lock] = None,
+        tmpdir_lock: Union[threading.Lock, None] = None,
     ) -> None:
         try:
             normalizeFilesDirs(self.builder.job)
@@ -201,7 +207,7 @@ class ExpressionTool(Process):
     def job(
         self,
         job_order: CWLObjectType,
-        output_callbacks: Optional[OutputCallbackType],
+        output_callbacks: OutputCallbackType | None,
         runtimeContext: RuntimeContext,
     ) -> Generator[ExpressionJob, None, None]:
         builder = self._init_job(job_order, runtimeContext)
@@ -221,7 +227,7 @@ class AbstractOperation(Process):
     def job(
         self,
         job_order: CWLObjectType,
-        output_callbacks: Optional[OutputCallbackType],
+        output_callbacks: OutputCallbackType | None,
         runtimeContext: RuntimeContext,
     ) -> JobsGeneratorType:
         raise WorkflowException("Abstract operation cannot be executed.")
@@ -233,7 +239,7 @@ def remove_path(f: CWLObjectType) -> None:
         del f["path"]
 
 
-def revmap_file(builder: Builder, outdir: str, f: CWLObjectType) -> Optional[CWLObjectType]:
+def revmap_file(builder: Builder, outdir: str, f: CWLObjectType) -> CWLObjectType | None:
     """
     Remap a file from internal path to external path.
 
@@ -312,7 +318,7 @@ class CallbackJob:
     def __init__(
         self,
         job: "CommandLineTool",
-        output_callback: Optional[OutputCallbackType],
+        output_callback: OutputCallbackType | None,
         cachebuilder: Builder,
         jobcache: str,
     ) -> None:
@@ -321,12 +327,12 @@ class CallbackJob:
         self.output_callback = output_callback
         self.cachebuilder = cachebuilder
         self.outdir = jobcache
-        self.prov_obj: Optional[ProvenanceProfile] = None
+        self.prov_obj: ProvenanceProfile | None = None
 
     def run(
         self,
         runtimeContext: RuntimeContext,
-        tmpdir_lock: Optional[threading.Lock] = None,
+        tmpdir_lock: Union[threading.Lock, None] = None,
     ) -> None:
         if self.output_callback:
             self.output_callback(
@@ -470,7 +476,7 @@ class CommandLineTool(Process):
         for ls in cast(list[CWLObjectType], fn.get("listing", [])):
             self.updatePathmap(os.path.join(outdir, cast(str, fn["basename"])), pathmap, ls)
 
-    def _initialworkdir(self, j: Optional[JobBase], builder: Builder) -> None:
+    def _initialworkdir(self, j: JobBase | None, builder: Builder) -> None:
         """
         Test and initialize the working directory.
 
@@ -797,7 +803,7 @@ class CommandLineTool(Process):
         job_order: CWLObjectType,
         output_callbacks: OutputCallbackType,
         runtimeContext: RuntimeContext,
-    ) -> Generator[Union[JobBase, CallbackJob], None, None]:
+    ) -> Generator[JobBase | CallbackJob, None, None]:
         workReuse, _ = self.get_requirement("WorkReuse")
         enableReuse = workReuse.get("enableReuse", True) if workReuse else True
 
@@ -840,15 +846,13 @@ class CommandLineTool(Process):
                 cmdline = ["docker", "run", dockerimg] + cmdline
                 # not really run using docker, just for hashing purposes
 
-            keydict: dict[str, Union[MutableSequence[Union[str, int]], CWLObjectType]] = {
-                "cmdline": cmdline
-            }
+            keydict: dict[str, MutableSequence[str | int] | CWLObjectType] = {"cmdline": cmdline}
 
             for shortcut in ["stdin", "stdout", "stderr"]:
                 if shortcut in self.tool:
                     keydict[shortcut] = self.tool[shortcut]
 
-            def calc_checksum(location: str) -> Optional[str]:
+            def calc_checksum(location: str) -> str | None:
                 for e in cachebuilder.files:
                     if (
                         "location" in e
@@ -943,7 +947,7 @@ class CommandLineTool(Process):
                 def update_status_output_callback(
                     output_callbacks: OutputCallbackType,
                     jobcachelock: TextIO,
-                    outputs: Optional[CWLObjectType],
+                    outputs: CWLObjectType | None,
                     processStatus: str,
                 ) -> None:
                     # save status to the lockfile then release the lock
@@ -1198,13 +1202,13 @@ class CommandLineTool(Process):
 
     def collect_output_ports(
         self,
-        ports: Union[CommentedSeq, set[CWLObjectType]],
+        ports: CommentedSeq | set[CWLObjectType],
         builder: Builder,
         outdir: str,
         rcode: int,
         compute_checksum: bool = True,
         jobname: str = "",
-        readers: Optional[MutableMapping[str, CWLObjectType]] = None,
+        readers: MutableMapping[str, CWLObjectType] | None = None,
     ) -> OutputPortsType:
         ret: OutputPortsType = {}
         debug = _logger.isEnabledFor(logging.DEBUG)
@@ -1212,6 +1216,7 @@ class CommandLineTool(Process):
         if cwl_version != "v1.0":
             builder.resources["exitCode"] = rcode
         try:
+            expected_schema = cast(RecordSchema, self.names.get_name("outputs_record_schema", None))
             fs_access = builder.make_fs_access(outdir)
             custom_output = fs_access.join(outdir, "cwl.output.json")
             if fs_access.exists(custom_output):
@@ -1223,6 +1228,20 @@ class CommandLineTool(Process):
                         custom_output,
                         json_dumps(ret, indent=4),
                     )
+                if ORDERED_VERSIONS.index(cast(str, cwl_version)) >= ORDERED_VERSIONS.index(
+                    "v1.3.0-dev1"
+                ):
+                    for k in list(ret):
+                        found = False
+                        for field in expected_schema.fields:
+                            if k == field.name:
+                                found = True
+                                break
+                        if not found:
+                            _logger.warning(
+                                f"Discarded undeclared output named {k!r} from {custom_output}."
+                            )
+                            ret.pop(k)
             else:
                 for i, port in enumerate(ports):
                     with SourceLine(
@@ -1253,7 +1272,6 @@ class CommandLineTool(Process):
 
                 if compute_checksum:
                     adjustFileObjs(ret, partial(compute_checksums, fs_access))
-            expected_schema = cast(Schema, self.names.get_name("outputs_record_schema", None))
             validate_ex(
                 expected_schema,
                 ret,
@@ -1280,11 +1298,11 @@ class CommandLineTool(Process):
         outdir: str,
         fs_access: StdFsAccess,
         compute_checksum: bool = True,
-    ) -> Optional[CWLOutputType]:
+    ) -> CWLOutputType | None:
         r: list[CWLOutputType] = []
         empty_and_optional = False
         debug = _logger.isEnabledFor(logging.DEBUG)
-        result: Optional[CWLOutputType] = None
+        result: CWLOutputType | None = None
         if "outputBinding" in schema:
             binding = cast(
                 MutableMapping[str, Union[bool, str, list[str]]],
@@ -1384,13 +1402,14 @@ class CommandLineTool(Process):
 
             optional = False
             single = False
-            if isinstance(schema["type"], MutableSequence):
-                if "null" in schema["type"]:
-                    optional = True
-                if "File" in schema["type"] or "Directory" in schema["type"]:
+            match schema["type"]:
+                case MutableSequence():
+                    if "null" in schema["type"]:
+                        optional = True
+                    if "File" in schema["type"] or "Directory" in schema["type"]:
+                        single = True
+                case "File" | "Directory":
                     single = True
-            elif schema["type"] == "File" or schema["type"] == "Directory":
-                single = True
 
             if "outputEval" in binding:
                 with SourceLine(binding, "outputEval", WorkflowException, debug):
