@@ -5,8 +5,8 @@ import os
 import shutil
 import tempfile
 import threading
-from collections.abc import Iterable
-from typing import IO, TYPE_CHECKING, Any, Callable, Literal, Optional, TextIO, Union
+from collections.abc import Callable, Iterable
+from typing import IO, TYPE_CHECKING, Any, Literal, Optional, TextIO, Union
 
 from ruamel.yaml.comments import CommentedMap
 from schema_salad.avro.schema import Names
@@ -35,7 +35,7 @@ if TYPE_CHECKING:
 class ContextBase:
     """Shared kwargs based initializer for :py:class:`RuntimeContext` and :py:class:`LoadingContext`."""
 
-    def __init__(self, kwargs: Optional[dict[str, Any]] = None) -> None:
+    def __init__(self, kwargs: dict[str, Any] | None = None) -> None:
         """Initialize."""
         if kwargs:
             for k, v in kwargs.items():
@@ -54,8 +54,8 @@ default_make_tool = make_tool_notimpl
 def log_handler(
     outdir: str,
     base_path_logs: str,
-    stdout_path: Optional[str],
-    stderr_path: Optional[str],
+    stdout_path: str | None,
+    stderr_path: str | None,
 ) -> None:
     """Move logs from log location to final output."""
     if outdir != base_path_logs:
@@ -76,31 +76,31 @@ def set_log_dir(outdir: str, log_dir: str, subdir_name: str) -> str:
 
 
 class LoadingContext(ContextBase):
-    def __init__(self, kwargs: Optional[dict[str, Any]] = None) -> None:
+    def __init__(self, kwargs: dict[str, Any] | None = None) -> None:
         """Initialize the LoadingContext from the kwargs."""
         self.debug: bool = False
         self.metadata: CWLObjectType = {}
-        self.requirements: Optional[list[CWLObjectType]] = None
-        self.hints: Optional[list[CWLObjectType]] = None
+        self.requirements: list[CWLObjectType] | None = None
+        self.hints: list[CWLObjectType] | None = None
         self.overrides_list: list[CWLObjectType] = []
-        self.loader: Optional[Loader] = None
-        self.avsc_names: Optional[Names] = None
+        self.loader: Loader | None = None
+        self.avsc_names: Names | None = None
         self.disable_js_validation: bool = False
-        self.js_hint_options_file: Optional[str] = None
+        self.js_hint_options_file: str | None = None
         self.do_validate: bool = True
         self.enable_dev: bool = False
         self.strict: bool = True
-        self.resolver: Optional[ResolverType] = None
-        self.fetcher_constructor: Optional[FetcherCallableType] = None
+        self.resolver: ResolverType | None = None
+        self.fetcher_constructor: FetcherCallableType | None = None
         self.construct_tool_object = default_make_tool
-        self.research_obj: Optional[ResearchObject] = None
+        self.research_obj: ResearchObject | None = None
         self.orcid: str = ""
         self.cwl_full_name: str = ""
         self.host_provenance: bool = False
         self.user_provenance: bool = False
         self.prov_obj: Optional["ProvenanceProfile"] = None
-        self.do_update: Optional[bool] = None
-        self.jobdefaults: Optional[CommentedMap] = None
+        self.do_update: bool | None = None
+        self.jobdefaults: CommentedMap | None = None
         self.doc_cache: bool = True
         self.relax_path_checks: bool = False
         self.singularity: bool = False
@@ -119,24 +119,24 @@ class LoadingContext(ContextBase):
 
 
 class RuntimeContext(ContextBase):
-    outdir: Optional[str] = None
+    outdir: str | None = None
     tmpdir: str = ""
     tmpdir_prefix: str = DEFAULT_TMP_PREFIX
     tmp_outdir_prefix: str = ""
     stagedir: str = ""
 
-    def __init__(self, kwargs: Optional[dict[str, Any]] = None) -> None:
+    def __init__(self, kwargs: dict[str, Any] | None = None) -> None:
         """Initialize the RuntimeContext from the kwargs."""
         select_resources_callable = Callable[
             [dict[str, Union[int, float]], RuntimeContext],
             dict[str, Union[int, float]],
         ]
-        self.user_space_docker_cmd: Optional[str] = None
+        self.user_space_docker_cmd: str | None = None
         self.secret_store: Optional["SecretStore"] = None
         self.no_read_only: bool = False
-        self.custom_net: Optional[str] = None
+        self.custom_net: str | None = None
         self.no_match_user: bool = False
-        self.preserve_environment: Optional[Iterable[str]] = None
+        self.preserve_environment: Iterable[str] | None = None
         self.preserve_entire_environment: bool = False
         self.use_container: bool = True
         self.force_docker_pull: bool = False
@@ -144,20 +144,21 @@ class RuntimeContext(ContextBase):
         self.rm_tmpdir: bool = True
         self.pull_image: bool = True
         self.rm_container: bool = True
-        self.move_outputs: Union[Literal["move"], Literal["leave"], Literal["copy"]] = "move"
+        self.move_outputs: Literal["move"] | Literal["leave"] | Literal["copy"] = "move"
         self.log_dir: str = ""
         self.set_log_dir = set_log_dir
         self.log_dir_handler = log_handler
         self.streaming_allowed: bool = False
 
         self.singularity: bool = False
+        self.image_base_path: str | None = None
         self.podman: bool = False
         self.debug: bool = False
         self.compute_checksum: bool = True
         self.name: str = ""
-        self.default_container: Optional[str] = ""
-        self.find_default_container: Optional[Callable[[HasReqsHints], Optional[str]]] = None
-        self.cachedir: Optional[str] = None
+        self.default_container: str | None = ""
+        self.find_default_container: Callable[[HasReqsHints], str | None] | None = None
+        self.cachedir: str | None = None
         self.part_of: str = ""
         self.basedir: str = ""
         self.toplevel: bool = False
@@ -169,32 +170,32 @@ class RuntimeContext(ContextBase):
         self.docker_tmpdir: str = ""
         self.docker_stagedir: str = ""
         self.js_console: bool = False
-        self.job_script_provider: Optional[DependenciesConfiguration] = None
-        self.select_resources: Optional[select_resources_callable] = None
+        self.job_script_provider: DependenciesConfiguration | None = None
+        self.select_resources: select_resources_callable | None = None
         self.eval_timeout: float = 60
-        self.postScatterEval: Optional[Callable[[CWLObjectType], Optional[CWLObjectType]]] = None
-        self.on_error: Union[Literal["stop"], Literal["continue"]] = "stop"
+        self.postScatterEval: Callable[[CWLObjectType], CWLObjectType | None] | None = None
+        self.on_error: Literal["stop"] | Literal["continue"] = "stop"
         self.strict_memory_limit: bool = False
         self.strict_cpu_limit: bool = False
-        self.cidfile_dir: Optional[str] = None
-        self.cidfile_prefix: Optional[str] = None
+        self.cidfile_dir: str | None = None
+        self.cidfile_prefix: str | None = None
 
-        self.workflow_eval_lock: Optional[threading.Condition] = None
-        self.research_obj: Optional[ResearchObject] = None
+        self.workflow_eval_lock: Union[threading.Condition, None] = None
+        self.research_obj: ResearchObject | None = None
         self.orcid: str = ""
         self.cwl_full_name: str = ""
-        self.process_run_id: Optional[str] = None
+        self.process_run_id: str | None = None
         self.prov_host: bool = False
         self.prov_user: bool = False
-        self.prov_obj: Optional[ProvenanceProfile] = None
+        self.prov_obj: ProvenanceProfile | None = None
         self.mpi_config: MpiConfig = MpiConfig()
-        self.default_stdout: Optional[Union[IO[bytes], TextIO]] = None
-        self.default_stderr: Optional[Union[IO[bytes], TextIO]] = None
+        self.default_stdout: IO[bytes] | TextIO | None = None
+        self.default_stderr: IO[bytes] | TextIO | None = None
         self.validate_only: bool = False
         self.validate_stdout: Optional["SupportsWrite[str]"] = None
-        self.workflow_job_step_name_callback: Optional[
+        self.workflow_job_step_name_callback: None | (
             Callable[[WorkflowJobStep, CWLObjectType], str]
-        ] = None
+        ) = None
 
         super().__init__(kwargs)
         if self.tmp_outdir_prefix == "":
