@@ -4,7 +4,8 @@ Our version of the popular flatten() method.
 http://rightfootin.blogspot.com/2006/09/more-on-python-flatten.html
 """
 
-from typing import Any, Callable, cast
+from collections.abc import Callable
+from typing import Any, cast
 
 
 def flatten(thing: Any) -> list[Any]:
