@@ -1,12 +1,11 @@
 import urllib
 from collections.abc import Mapping, MutableMapping, MutableSequence
-from typing import Any, NamedTuple, Optional, Union, cast
+from typing import Any, NamedTuple, Union, cast
 
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
 
 from .context import LoadingContext
 from .load_tool import load_tool, make_tool
-from .process import Process
 from .utils import CWLObjectType, aslist
 from .workflow import Workflow, WorkflowStep
 
@@ -14,7 +13,7 @@ from .workflow import Workflow, WorkflowStep
 class _Node(NamedTuple):
     up: list[str]
     down: list[str]
-    type: Optional[str]
+    type: str | None
 
 
 UP = "up"
@@ -42,7 +41,7 @@ def _subgraph_visit(
         _subgraph_visit(c, nodes, visited, direction)
 
 
-def _declare_node(nodes: dict[str, _Node], nodeid: str, tp: Optional[str]) -> _Node:
+def _declare_node(nodes: dict[str, _Node], nodeid: str, tp: str | None) -> _Node:
     """
     Record the given nodeid in the graph.
 
@@ -60,50 +59,50 @@ def _declare_node(nodes: dict[str, _Node], nodeid: str, tp: Optional[str]) -> _N
 
 def find_step(
     steps: list[WorkflowStep], stepid: str, loading_context: LoadingContext
-) -> tuple[Optional[CWLObjectType], Optional[WorkflowStep]]:
+) -> tuple[CWLObjectType | None, WorkflowStep | None]:
     """Find the step (raw dictionary and WorkflowStep) for a given step id."""
     for st in steps:
         st_tool_id = st.tool["id"]
         if st_tool_id == stepid:
             return st.tool, st
         if stepid.startswith(st_tool_id):
-            run: Union[str, Process, CWLObjectType] = st.tool["run"]
-            if isinstance(run, Workflow):
-                result, st2 = find_step(
-                    run.steps, stepid[len(st.tool["id"]) + 1 :], loading_context
-                )
-                if result:
-                    return result, st2
-            elif isinstance(run, CommentedMap) and run["class"] == "Workflow":
-                process = make_tool(run, loading_context)
-                if isinstance(process, Workflow):
-                    suffix = stepid[len(st.tool["id"]) + 1 :]
-                    prefix = process.tool["id"]
-                    if "#" in prefix:
-                        sep = "/"
-                    else:
-                        sep = "#"
-                    adj_stepid = f"{prefix}{sep}{suffix}"
-                    result2, st3 = find_step(
-                        process.steps,
-                        adj_stepid,
-                        loading_context,
+            match st.tool["run"]:
+                case Workflow(steps=steps):
+                    result, st2 = find_step(
+                        steps, stepid[len(st.tool["id"]) + 1 :], loading_context
                     )
-                    if result2:
-                        return result2, st3
-            elif isinstance(run, str):
-                process = load_tool(run, loading_context)
-                if isinstance(process, Workflow):
-                    suffix = stepid[len(st.tool["id"]) + 1 :]
-                    prefix = process.tool["id"]
-                    if "#" in prefix:
-                        sep = "/"
-                    else:
-                        sep = "#"
-                    adj_stepid = f"{prefix}{sep}{suffix}"
-                    result3, st4 = find_step(process.steps, adj_stepid, loading_context)
-                    if result3:
-                        return result3, st4
+                    if result:
+                        return result, st2
+                case {"class": "Workflow"}:
+                    process = make_tool(st.tool["run"], loading_context)
+                    if isinstance(process, Workflow):
+                        suffix = stepid[len(st.tool["id"]) + 1 :]
+                        prefix = process.tool["id"]
+                        if "#" in prefix:
+                            sep = "/"
+                        else:
+                            sep = "#"
+                        adj_stepid = f"{prefix}{sep}{suffix}"
+                        result2, st3 = find_step(
+                            process.steps,
+                            adj_stepid,
+                            loading_context,
+                        )
+                        if result2:
+                            return result2, st3
+                case str(run_line):
+                    process = load_tool(run_line, loading_context)
+                    if isinstance(process, Workflow):
+                        suffix = stepid[len(st.tool["id"]) + 1 :]
+                        prefix = process.tool["id"]
+                        if "#" in prefix:
+                            sep = "/"
+                        else:
+                            sep = "#"
+                        adj_stepid = f"{prefix}{sep}{suffix}"
+                        result3, st4 = find_step(process.steps, adj_stepid, loading_context)
+                        if result3:
+                            return result3, st4
     return None, None
 
 
@@ -264,7 +263,7 @@ def get_process(
     if raw_step is None or step is None:
         raise Exception(f"Step {step_id} was not found")
 
-    run: Union[str, Any] = raw_step["run"]
+    run: str | Any = raw_step["run"]
 
     if isinstance(run, str):
         process = loading_context.loader.idx[run]
