@@ -4,7 +4,7 @@ import json
 import logging
 from collections.abc import MutableMapping, MutableSequence
 from importlib.resources import files
-from typing import Any, NamedTuple, Optional, Union, cast
+from typing import Any, NamedTuple, cast
 
 from cwl_utils.errors import SubstitutionError
 from cwl_utils.expression import scanner as scan_expression
@@ -25,7 +25,7 @@ from .errors import WorkflowException
 from .loghandler import _logger
 
 
-def is_expression(tool: Any, schema: Optional[Schema]) -> bool:
+def is_expression(tool: Any, schema: Schema | None) -> bool:
     """Test a field/schema combo to see if it is a CWL Expression."""
     return (
         isinstance(schema, EnumSchema)
@@ -50,63 +50,62 @@ _logger_validation_warnings.addFilter(SuppressLog("cwltool.validation_warnings")
 
 
 def get_expressions(
-    tool: Union[CommentedMap, str, CommentedSeq],
-    schema: Optional[Union[Schema, ArraySchema]],
-    source_line: Optional[SourceLine] = None,
-) -> list[tuple[str, Optional[SourceLine]]]:
+    tool: CommentedMap | str | CommentedSeq,
+    schema: Schema | ArraySchema | None,
+    source_line: SourceLine | None = None,
+) -> list[tuple[str, SourceLine | None]]:
     debug = _logger.isEnabledFor(logging.DEBUG)
     if is_expression(tool, schema):
         return [(cast(str, tool), source_line)]
-    elif isinstance(schema, UnionSchema):
-        valid_schema = None
+    match schema:
+        case UnionSchema(schemas=schemas):
+            valid_schema = None
 
-        for possible_schema in schema.schemas:
-            if is_expression(tool, possible_schema):
-                return [(cast(str, tool), source_line)]
-            elif validate_ex(
-                possible_schema,
-                tool,
-                raise_ex=False,
-                logger=_logger_validation_warnings,
-                vocab={},
-            ):
-                valid_schema = possible_schema
+            for possible_schema in schemas:
+                if is_expression(tool, possible_schema):
+                    return [(cast(str, tool), source_line)]
+                elif validate_ex(
+                    possible_schema,
+                    tool,
+                    raise_ex=False,
+                    logger=_logger_validation_warnings,
+                    vocab={},
+                ):
+                    valid_schema = possible_schema
 
-        return get_expressions(tool, valid_schema, source_line)
-    elif isinstance(schema, ArraySchema):
-        if not isinstance(tool, MutableSequence):
-            return []
+            return get_expressions(tool, valid_schema, source_line)
+        case ArraySchema(items=items):
+            if not isinstance(tool, MutableSequence):
+                return []
 
-        return list(
-            itertools.chain(
-                *map(
-                    lambda x: get_expressions(
-                        x[1], getattr(schema, "items"), SourceLine(tool, x[0])  # noqa: B009
-                    ),
-                    enumerate(tool),
-                )
-            )
-        )
-
-    elif isinstance(schema, RecordSchema):
-        if not isinstance(tool, MutableMapping):
-            return []
-
-        expression_nodes = []
-
-        for schema_field in schema.fields:
-            if schema_field.name in tool:
-                expression_nodes.extend(
-                    get_expressions(
-                        tool[schema_field.name],
-                        schema_field.type,
-                        SourceLine(tool, schema_field.name, include_traceback=debug),
+            return list(
+                itertools.chain(
+                    *map(
+                        lambda x: get_expressions(
+                            x[1], items, SourceLine(tool, x[0])  # noqa: B009
+                        ),
+                        enumerate(tool),
                     )
                 )
+            )
+        case RecordSchema(fields=fields):
+            if not isinstance(tool, MutableMapping):
+                return []
 
-        return expression_nodes
-    else:
-        return []
+            expression_nodes = []
+
+            for schema_field in fields:
+                if schema_field.name in tool:
+                    expression_nodes.extend(
+                        get_expressions(
+                            tool[schema_field.name],
+                            schema_field.type,
+                            SourceLine(tool, schema_field.name, include_traceback=debug),
+                        )
+                    )
+            return expression_nodes
+        case _:
+            return []
 
 
 class JSHintJSReturn(NamedTuple):
@@ -118,8 +117,8 @@ class JSHintJSReturn(NamedTuple):
 
 def jshint_js(
     js_text: str,
-    globals: Optional[list[str]] = None,
-    options: Optional[dict[str, Union[list[str], str, int]]] = None,
+    globals: list[str] | None = None,
+    options: dict[str, list[str] | str | int] | None = None,
     container_engine: str = "docker",
     eval_timeout: float = 60,
 ) -> JSHintJSReturn:
@@ -187,7 +186,7 @@ def jshint_js(
     return JSHintJSReturn(jshint_errors, jshint_json.get("globals", []))
 
 
-def print_js_hint_messages(js_hint_messages: list[str], source_line: Optional[SourceLine]) -> None:
+def print_js_hint_messages(js_hint_messages: list[str], source_line: SourceLine | None) -> None:
     """Log the message from JSHint, using the line number."""
     if source_line is not None:
         for js_hint_message in js_hint_messages:
@@ -197,7 +196,7 @@ def print_js_hint_messages(js_hint_messages: list[str], source_line: Optional[So
 def validate_js_expressions(
     tool: CommentedMap,
     schema: Schema,
-    jshint_options: Optional[dict[str, Union[list[str], str, int]]] = None,
+    jshint_options: dict[str, list[str] | str | int] | None = None,
     container_engine: str = "docker",
     eval_timeout: float = 60,
 ) -> None:
