@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 from prov.identifier import Namespace
 
 __author__ = "Trung Dong Huynh"
@@ -170,13 +168,9 @@ PROV_ATTRIBUTE_LITERALS = {PROV_ATTR_TIME, PROV_ATTR_STARTTIME, PROV_ATTR_ENDTIM
 PROV_ATTRIBUTES = PROV_ATTRIBUTE_QNAMES | PROV_ATTRIBUTE_LITERALS
 PROV_RECORD_ATTRIBUTES = list((attr, str(attr)) for attr in PROV_ATTRIBUTES)
 
-PROV_RECORD_IDS_MAP = dict((PROV_N_MAP[rec_type_id], rec_type_id) for rec_type_id in PROV_N_MAP)
-PROV_ID_ATTRIBUTES_MAP = dict(
-    (prov_id, attribute) for (prov_id, attribute) in PROV_RECORD_ATTRIBUTES
-)
-PROV_ATTRIBUTES_ID_MAP = dict(
-    (attribute, prov_id) for (prov_id, attribute) in PROV_RECORD_ATTRIBUTES
-)
+PROV_RECORD_IDS_MAP = {PROV_N_MAP[rec_type_id]: rec_type_id for rec_type_id in PROV_N_MAP}
+PROV_ID_ATTRIBUTES_MAP = {prov_id: attribute for (prov_id, attribute) in PROV_RECORD_ATTRIBUTES}
+PROV_ATTRIBUTES_ID_MAP = {attribute: prov_id for (prov_id, attribute) in PROV_RECORD_ATTRIBUTES}
 
 # Extra definition for convenience
 PROV_TYPE = PROV["type"]
