@@ -191,6 +191,46 @@ def test_loop_value_from_fail_no_requirement() -> None:
     assert main(params) == 1
 
 
+def test_loop_value_from_literal_file() -> None:
+    """Test a loop case with a literal file generated by a valueFrom directive inside the loop."""
+    stream = StringIO()
+    params = [
+        "--enable-dev",
+        get_data("tests/loop/value-from-loop-2.cwl"),
+    ]
+    main(params, stdout=stream)
+    expected = {
+        "result": [
+            "first,0-sha1$e445e7ebf0b1e19e5eb7f3e8e4d86f5424c4b103",
+            "first,1-sha1$c2ee1cb0abf988529ae1d67c5a6798a9db9e133a",
+            "first,2-sha1$7663c0076cdf1dbe14373237e8c2eea8735b04f2",
+            "first,3-sha1$83b77de68c3c6cce92a381adc9159d6e59332545",
+            "first,4-sha1$eb74efd51e4399144f465534f4077f515d996d57",
+        ]
+    }
+    assert json.loads(stream.getvalue()) == expected
+
+
+def test_loop_and_value_from_literal_file() -> None:
+    """Test a loop case with a literal file generated by a valueFrom directive outside the loop."""
+    stream = StringIO()
+    params = [
+        "--enable-dev",
+        get_data("tests/loop/value-from-loop-3.cwl"),
+    ]
+    main(params, stdout=stream)
+    expected = {
+        "result": [
+            "first,0-sha1$e445e7ebf0b1e19e5eb7f3e8e4d86f5424c4b103",
+            "first,0-sha1$e445e7ebf0b1e19e5eb7f3e8e4d86f5424c4b103",
+            "first,0-sha1$e445e7ebf0b1e19e5eb7f3e8e4d86f5424c4b103",
+            "first,0-sha1$e445e7ebf0b1e19e5eb7f3e8e4d86f5424c4b103",
+            "first,0-sha1$e445e7ebf0b1e19e5eb7f3e8e4d86f5424c4b103",
+        ]
+    }
+    assert json.loads(stream.getvalue()) == expected
+
+
 def test_loop_inside_scatter() -> None:
     """Test a loop subworkflow inside a scatter step."""
     stream = StringIO()
