@@ -11,7 +11,7 @@ from .util import get_data, needs_docker
 @needs_docker
 def test_sequential_workflow(tmp_path: Path) -> None:
     test_file = "tests/wf/count-lines1-wf.cwl"
-    executor = MultithreadedJobExecutor()
+    executor = MultithreadedJobExecutor(max_parallel=0)
     runtime_context = RuntimeContext()
     runtime_context.outdir = str(tmp_path)
     runtime_context.select_resources = executor.select_resources
@@ -25,7 +25,7 @@ def test_sequential_workflow(tmp_path: Path) -> None:
 def test_scattered_workflow() -> None:
     test_file = "tests/wf/scatter-wf4.cwl"
     job_file = "tests/wf/scatter-job2.json"
-    factory = Factory(MultithreadedJobExecutor())
+    factory = Factory(MultithreadedJobExecutor(max_parallel=0))
     echo = factory.make(get_data(test_file))
     with open(get_data(job_file)) as job:
         assert echo(**json.load(job)) == {"out": ["foo one three", "foo two four"]}
